@@ -1,5 +1,14 @@
-from dataclasses import dataclass, field
-from typing import TYPE_CHECKING, Iterable, List, NamedTuple, Optional, Tuple, Union
+from dataclasses import dataclass, field, replace
+from typing import (
+    Dict,
+    TYPE_CHECKING,
+    Iterable,
+    List,
+    NamedTuple,
+    Optional,
+    Tuple,
+    Union,
+)
 
 from . import box, errors
 from ._loop import loop_first_last, loop_last
@@ -11,7 +20,6 @@ from .padding import Padding, PaddingDimensions
 from .protocol import is_renderable
 from .segment import Segment
 from .style import Style, StyleType
-from .styled import Styled
 from .text import Text, TextType
 
 if TYPE_CHECKING:
@@ -35,13 +43,13 @@ class Column:
     footer: "RenderableType" = ""
     """RenderableType: Renderable for the footer (typically a string)"""
 
-    header_style: StyleType = "table.header"
+    header_style: StyleType = ""
     """StyleType: The style of the header."""
 
-    footer_style: StyleType = "table.footer"
+    footer_style: StyleType = ""
     """StyleType: The style of the footer."""
 
-    style: StyleType = "none"
+    style: StyleType = ""
     """StyleType: The style of the column."""
 
     justify: "JustifyMethod" = "left"
@@ -70,6 +78,10 @@ class Column:
 
     _cells: List["RenderableType"] = field(default_factory=list)
 
+    def copy(self) -> "Column":
+        """Return a copy of this Column."""
+        return replace(self, _cells=[])
+
     @property
     def cells(self) -> Iterable["RenderableType"]:
         """Get all cells in the column, not including header."""
@@ -81,6 +93,17 @@ class Column:
         return self.ratio is not None
 
 
+@dataclass
+class Row:
+    """Information regarding a row."""
+
+    style: Optional[StyleType] = None
+    """Style to apply to row."""
+
+    end_section: bool = False
+    """Indicated end of section, which will force a line beneath the row."""
+
+
 class _Cell(NamedTuple):
     """A single cell in a table."""
 
@@ -112,24 +135,26 @@ class Table(JupyterMixin):
         leading (bool, optional): Number of blank lines between rows (precludes ``show_lines``). Defaults to 0.
         style (Union[str, Style], optional): Default style for the table. Defaults to "none".
         row_styles (List[Union, str], optional): Optional list of row styles, if more that one style is give then the styles will alternate. Defaults to None.
-        header_style (Union[str, Style], optional): Style of the header. Defaults to None.
-        footer_style (Union[str, Style], optional): Style of the footer. Defaults to None.
+        header_style (Union[str, Style], optional): Style of the header. Defaults to "table.header".
+        footer_style (Union[str, Style], optional): Style of the footer. Defaults to "table.footer".
         border_style (Union[str, Style], optional): Style of the border. Defaults to None.
         title_style (Union[str, Style], optional): Style of the title. Defaults to None.
         caption_style (Union[str, Style], optional): Style of the caption. Defaults to None.
         title_justify (str, optional): Justify method for title. Defaults to "center".
         caption_justify (str, optional): Justify method for caption. Defaults to "center".
+        highlight (bool, optional): Highlight cell contents (if str). Defaults to False.
     """
 
     columns: List[Column]
+    rows: List[Row]
 
     def __init__(
         self,
         *headers: Union[Column, str],
-        title: TextType = None,
-        caption: TextType = None,
-        width: int = None,
-        min_width: int = None,
+        title: Optional[TextType] = None,
+        caption: Optional[TextType] = None,
+        width: Optional[int] = None,
+        min_width: Optional[int] = None,
         box: Optional[box.Box] = box.HEAVY_HEAD,
         safe_box: Optional[bool] = None,
         padding: PaddingDimensions = (0, 1),
@@ -142,25 +167,19 @@ class Table(JupyterMixin):
         show_lines: bool = False,
         leading: int = 0,
         style: StyleType = "none",
-        row_styles: Iterable[StyleType] = None,
-        header_style: StyleType = None,
-        footer_style: StyleType = None,
-        border_style: StyleType = None,
-        title_style: StyleType = None,
-        caption_style: StyleType = None,
+        row_styles: Optional[Iterable[StyleType]] = None,
+        header_style: Optional[StyleType] = "table.header",
+        footer_style: Optional[StyleType] = "table.footer",
+        border_style: Optional[StyleType] = None,
+        title_style: Optional[StyleType] = None,
+        caption_style: Optional[StyleType] = None,
         title_justify: "JustifyMethod" = "center",
         caption_justify: "JustifyMethod" = "center",
+        highlight: bool = False,
     ) -> None:
 
         self.columns: List[Column] = []
-        append_column = self.columns.append
-        for index, header in enumerate(headers):
-            if isinstance(header, str):
-                append_column(Column(_index=index, header=header))
-            else:
-                header._index = index
-                append_column(header)
-
+        self.rows: List[Row] = []
         self.title = title
         self.caption = caption
         self.width = width
@@ -177,19 +196,27 @@ class Table(JupyterMixin):
         self.leading = leading
         self.collapse_padding = collapse_padding
         self.style = style
-        self.header_style = header_style
-        self.footer_style = footer_style
+        self.header_style = header_style or ""
+        self.footer_style = footer_style or ""
         self.border_style = border_style
         self.title_style = title_style
         self.caption_style = caption_style
         self.title_justify = title_justify
         self.caption_justify = caption_justify
-        self._row_count = 0
+        self.highlight = highlight
         self.row_styles = list(row_styles or [])
+        append_column = self.columns.append
+        for header in headers:
+            if isinstance(header, str):
+                self.add_column(header=header)
+            else:
+                header._index = len(self.columns)
+                append_column(header)
 
     @classmethod
     def grid(
         cls,
+        *headers: Union[Column, str],
         padding: PaddingDimensions = 0,
         collapse_padding: bool = True,
         pad_edge: bool = False,
@@ -198,6 +225,7 @@ class Table(JupyterMixin):
         """Get a table with no lines, headers, or footer.
 
         Args:
+            *headers (Union[Column, str]): Column headers, either as a string, or :class:`~rich.table.Column` instance.
             padding (PaddingDimensions, optional): Get padding around cells. Defaults to 0.
             collapse_padding (bool, optional): Enable collapsing of padding around cells. Defaults to True.
             pad_edge (bool, optional): Enable padding around edges of table. Defaults to False.
@@ -207,6 +235,7 @@ class Table(JupyterMixin):
             Table: A table instance.
         """
         return cls(
+            *headers,
             box=None,
             padding=padding,
             collapse_padding=collapse_padding,
@@ -240,28 +269,38 @@ class Table(JupyterMixin):
     @property
     def row_count(self) -> int:
         """Get the current number of rows."""
-        return self._row_count
+        return len(self.rows)
 
-    def get_row_style(self, index: int) -> StyleType:
+    def get_row_style(self, console: "Console", index: int) -> StyleType:
         """Get the current row style."""
+        style = Style.null()
         if self.row_styles:
-            return self.row_styles[index % len(self.row_styles)]
-        return Style.null()
+            style += console.get_style(self.row_styles[index % len(self.row_styles)])
+        row_style = self.rows[index].style
+        if row_style is not None:
+            style += console.get_style(row_style)
+        return style
 
-    def __rich_measure__(self, console: "Console", max_width: int) -> Measurement:
+    def __rich_measure__(
+        self, console: "Console", options: "ConsoleOptions"
+    ) -> Measurement:
+        max_width = options.max_width
         if self.width is not None:
             max_width = self.width
         if max_width < 0:
             return Measurement(0, 0)
 
         extra_width = self._extra_width
-
-        max_width = sum(self._calculate_column_widths(console, max_width - extra_width))
-
+        max_width = sum(
+            self._calculate_column_widths(
+                console, options.update_width(max_width - extra_width)
+            )
+        )
         _measure_column = self._measure_column
 
         measurements = [
-            _measure_column(console, column, max_width) for column in self.columns
+            _measure_column(console, options.update_width(max_width), column)
+            for column in self.columns
         ]
         minimum_width = (
             sum(measurement.minimum for measurement in measurements) + extra_width
@@ -291,15 +330,15 @@ class Table(JupyterMixin):
         header: "RenderableType" = "",
         footer: "RenderableType" = "",
         *,
-        header_style: StyleType = None,
-        footer_style: StyleType = None,
-        style: StyleType = None,
+        header_style: Optional[StyleType] = None,
+        footer_style: Optional[StyleType] = None,
+        style: Optional[StyleType] = None,
         justify: "JustifyMethod" = "left",
         overflow: "OverflowMethod" = "ellipsis",
-        width: int = None,
-        min_width: int = None,
-        max_width: int = None,
-        ratio: int = None,
+        width: Optional[int] = None,
+        min_width: Optional[int] = None,
+        max_width: Optional[int] = None,
+        ratio: Optional[int] = None,
         no_wrap: bool = False,
     ) -> None:
         """Add a column to the table.
@@ -309,10 +348,11 @@ class Table(JupyterMixin):
                 Defaults to "".
             footer (RenderableType, optional): Text or renderable for the footer.
                 Defaults to "".
-            header_style (Union[str, Style], optional): Style for the header. Defaults to "none".
-            footer_style (Union[str, Style], optional): Style for the header. Defaults to "none".
-            style (Union[str, Style], optional): Style for the column cells. Defaults to "none".
+            header_style (Union[str, Style], optional): Style for the header, or None for default. Defaults to None.
+            footer_style (Union[str, Style], optional): Style for the footer, or None for default. Defaults to None.
+            style (Union[str, Style], optional): Style for the column cells, or None for default. Defaults to None.
             justify (JustifyMethod, optional): Alignment for cells. Defaults to "left".
+            overflow (OverflowMethod): Overflow method: "crop", "fold", "ellipsis". Defaults to "ellipsis".
             width (int, optional): Desired width of column in characters, or None to fit to contents. Defaults to None.
             min_width (Optional[int], optional): Minimum width of column, or ``None`` for no minimum. Defaults to None.
             max_width (Optional[int], optional): Maximum width of column, or ``None`` for no maximum. Defaults to None.
@@ -324,13 +364,9 @@ class Table(JupyterMixin):
             _index=len(self.columns),
             header=header,
             footer=footer,
-            header_style=Style.pick_first(
-                header_style, self.header_style, "table.header"
-            ),
-            footer_style=Style.pick_first(
-                footer_style, self.footer_style, "table.footer"
-            ),
-            style=Style.pick_first(style, self.style, "table.cell"),
+            header_style=header_style or "",
+            footer_style=footer_style or "",
+            style=style or "",
             justify=justify,
             overflow=overflow,
             width=width,
@@ -342,7 +378,10 @@ class Table(JupyterMixin):
         self.columns.append(column)
 
     def add_row(
-        self, *renderables: Optional["RenderableType"], style: StyleType = None
+        self,
+        *renderables: Optional["RenderableType"],
+        style: Optional[StyleType] = None,
+        end_section: bool = False,
     ) -> None:
         """Add a row of renderables.
 
@@ -350,15 +389,14 @@ class Table(JupyterMixin):
             *renderables (None or renderable): Each cell in a row must be a renderable object (including str),
                 or ``None`` for a blank cell.
             style (StyleType, optional): An optional style to apply to the entire row. Defaults to None.
+            end_section (bool, optional): End a section and draw a line. Defaults to False.
 
         Raises:
             errors.NotRenderableError: If you add something that can't be rendered.
         """
 
         def add_cell(column: Column, renderable: "RenderableType") -> None:
-            column._cells.append(
-                renderable if style is None else Styled(renderable, style)
-            )
+            column._cells.append(renderable)
 
         cell_renderables: List[Optional["RenderableType"]] = list(renderables)
 
@@ -371,7 +409,7 @@ class Table(JupyterMixin):
         for index, renderable in enumerate(cell_renderables):
             if index == len(columns):
                 column = Column(_index=index)
-                for _ in range(self._row_count):
+                for _ in self.rows:
                     add_cell(column, Text(""))
                 self.columns.append(column)
             else:
@@ -384,27 +422,37 @@ class Table(JupyterMixin):
                 raise errors.NotRenderableError(
                     f"unable to render {type(renderable).__name__}; a string or other renderable object is required"
                 )
-        self._row_count += 1
+        self.rows.append(Row(style=style, end_section=end_section))
 
     def __rich_console__(
         self, console: "Console", options: "ConsoleOptions"
     ) -> "RenderResult":
 
+        if not self.columns:
+            yield Segment("\n")
+            return
+
         max_width = options.max_width
         if self.width is not None:
             max_width = self.width
 
         extra_width = self._extra_width
-        widths = self._calculate_column_widths(console, max_width - extra_width)
+        widths = self._calculate_column_widths(
+            console, options.update_width(max_width - extra_width)
+        )
         table_width = sum(widths) + extra_width
 
-        render_options = options.update(width=table_width)
+        render_options = options.update(
+            width=table_width, highlight=self.highlight, height=None
+        )
 
         def render_annotation(
             text: TextType, style: StyleType, justify: "JustifyMethod" = "center"
         ) -> "RenderResult":
             render_text = (
-                console.render_str(text, style=style) if isinstance(text, str) else text
+                console.render_str(text, style=style, highlight=False)
+                if isinstance(text, str)
+                else text
             )
             return console.render(
                 render_text, options=render_options.update(justify=justify)
@@ -424,16 +472,18 @@ class Table(JupyterMixin):
                 justify=self.caption_justify,
             )
 
-    def _calculate_column_widths(self, console: "Console", max_width: int) -> List[int]:
+    def _calculate_column_widths(
+        self, console: "Console", options: "ConsoleOptions"
+    ) -> List[int]:
         """Calculate the widths of each column, including padding, not including borders."""
+        max_width = options.max_width
         columns = self.columns
         width_ranges = [
-            self._measure_column(console, column, max_width) for column in columns
+            self._measure_column(console, options, column) for column in columns
         ]
         widths = [_range.maximum or 1 for _range in width_ranges]
         get_padding_width = self._get_padding_width
         extra_width = self._extra_width
-
         if self.expand:
             ratios = [col.ratio or 0 for col in columns if col.flexible]
             if any(ratios):
@@ -461,7 +511,6 @@ class Table(JupyterMixin):
                 max_width,
             )
             table_width = sum(widths)
-
             # last resort, reduce columns evenly
             if table_width > max_width:
                 excess_width = table_width - max_width
@@ -469,10 +518,10 @@ class Table(JupyterMixin):
                 table_width = sum(widths)
 
             width_ranges = [
-                self._measure_column(console, column, width)
+                self._measure_column(console, options.update_width(width), column)
                 for width, column in zip(widths, columns)
             ]
-            widths = [_range.maximum or 1 for _range in width_ranges]
+            widths = [_range.maximum or 0 for _range in width_ranges]
 
         if (table_width < max_width and self.expand) or (
             self.min_width is not None and table_width < (self.min_width - extra_width)
@@ -526,7 +575,9 @@ class Table(JupyterMixin):
                 excess_width = total_width - max_width
         return widths
 
-    def _get_cells(self, column_index: int, column: Column) -> Iterable[_Cell]:
+    def _get_cells(
+        self, console: "Console", column_index: int, column: Column
+    ) -> Iterable[_Cell]:
         """Get all the cells with padding and optional header."""
 
         collapse_padding = self.collapse_padding
@@ -537,11 +588,12 @@ class Table(JupyterMixin):
         first_column = column_index == 0
         last_column = column_index == len(self.columns) - 1
 
-        def add_padding(
-            renderable: "RenderableType", first_row: bool, last_row: bool
-        ) -> "RenderableType":
-            if not any_padding:
-                return renderable
+        _padding_cache: Dict[Tuple[bool, bool], Tuple[int, int, int, int]] = {}
+
+        def get_padding(first_row: bool, last_row: bool) -> Tuple[int, int, int, int]:
+            cached = _padding_cache.get((first_row, last_row))
+            if cached:
+                return cached
             top, right, bottom, left = padding
 
             if collapse_padding:
@@ -559,19 +611,34 @@ class Table(JupyterMixin):
                     top = 0
                 if last_row:
                     bottom = 0
-            _padding = Padding(renderable, (top, right, bottom, left))
+            _padding = (top, right, bottom, left)
+            _padding_cache[(first_row, last_row)] = _padding
             return _padding
 
         raw_cells: List[Tuple[StyleType, "RenderableType"]] = []
         _append = raw_cells.append
+        get_style = console.get_style
         if self.show_header:
-            _append((column.header_style, column.header))
+            header_style = get_style(self.header_style or "") + get_style(
+                column.header_style
+            )
+            _append((header_style, column.header))
+        cell_style = get_style(column.style or "")
         for cell in column.cells:
-            _append((column.style, cell))
+            _append((cell_style, cell))
         if self.show_footer:
-            _append((column.footer_style, column.footer))
-        for first, last, (style, renderable) in loop_first_last(raw_cells):
-            yield _Cell(style, add_padding(renderable, first, last))
+            footer_style = get_style(self.footer_style or "") + get_style(
+                column.footer_style
+            )
+            _append((footer_style, column.footer))
+
+        if any_padding:
+            _Padding = Padding
+            for first, last, (style, renderable) in loop_first_last(raw_cells):
+                yield _Cell(style, _Padding(renderable, get_padding(first, last)))
+        else:
+            for (style, renderable) in raw_cells:
+                yield _Cell(style, renderable)
 
     def _get_padding_width(self, column_index: int) -> int:
         """Get extra width from padding."""
@@ -582,10 +649,14 @@ class Table(JupyterMixin):
         return pad_left + pad_right
 
     def _measure_column(
-        self, console: "Console", column: Column, max_width: int
+        self,
+        console: "Console",
+        options: "ConsoleOptions",
+        column: Column,
     ) -> Measurement:
         """Get the minimum and maximum width of the column."""
 
+        max_width = options.max_width
         if max_width < 1:
             return Measurement(0, 0)
 
@@ -602,8 +673,8 @@ class Table(JupyterMixin):
         append_min = min_widths.append
         append_max = max_widths.append
         get_render_width = Measurement.get
-        for cell in self._get_cells(column._index, column):
-            _min, _max = get_render_width(console, cell.renderable, max_width)
+        for cell in self._get_cells(console, column._index, column):
+            _min, _max = get_render_width(console, options, cell.renderable)
             append_min(_min)
             append_max(_max)
 
@@ -623,14 +694,11 @@ class Table(JupyterMixin):
         table_style = console.get_style(self.style or "")
 
         border_style = table_style + console.get_style(self.border_style or "")
-        rows: List[Tuple[_Cell, ...]] = list(
-            zip(
-                *(
-                    self._get_cells(column_index, column)
-                    for column_index, column in enumerate(self.columns)
-                )
-            )
+        _column_cells = (
+            self._get_cells(console, column_index, column)
+            for column_index, column in enumerate(self.columns)
         )
+        row_cells: List[Tuple[_Cell, ...]] = list(zip(*_column_cells))
         _box = (
             self.box.substitute(
                 options, safe=pick_bool(self.safe_box, console.safe_box)
@@ -677,23 +745,29 @@ class Table(JupyterMixin):
         get_row_style = self.get_row_style
         get_style = console.get_style
 
-        for index, (first, last, row) in enumerate(loop_first_last(rows)):
+        for index, (first, last, row_cell) in enumerate(loop_first_last(row_cells)):
             header_row = first and show_header
             footer_row = last and show_footer
+            row = (
+                self.rows[index - show_header]
+                if (not header_row and not footer_row)
+                else None
+            )
             max_height = 1
             cells: List[List[List[Segment]]] = []
             if header_row or footer_row:
                 row_style = Style.null()
             else:
                 row_style = get_style(
-                    get_row_style(index - 1 if show_header else index)
+                    get_row_style(console, index - 1 if show_header else index)
                 )
-            for width, cell, column in zip(widths, row, columns):
+            for width, cell, column in zip(widths, row_cell, columns):
                 render_options = options.update(
                     width=width,
                     justify=column.justify,
                     no_wrap=column.no_wrap,
                     overflow=column.overflow,
+                    height=None,
                 )
                 cell_style = table_style + row_style + get_style(cell.style)
                 lines = console.render_lines(
@@ -703,7 +777,9 @@ class Table(JupyterMixin):
                 cells.append(lines)
 
             cells[:] = [
-                _Segment.set_shape(_cell, width, max_height, style=table_style)
+                _Segment.set_shape(
+                    _cell, width, max_height, style=table_style + row_style
+                )
                 for width, _cell in zip(widths, cells)
             ]
 
@@ -743,18 +819,18 @@ class Table(JupyterMixin):
                     _box.get_row(widths, "head", edge=show_edge), border_style
                 )
                 yield new_line
-            if _box and (show_lines or leading):
+            end_section = row and row.end_section
+            if _box and (show_lines or leading or end_section):
                 if (
                     not last
-                    and not (show_footer and index >= len(rows) - 2)
+                    and not (show_footer and index >= len(row_cells) - 2)
                     and not (show_header and header_row)
                 ):
                     if leading:
-                        for _ in range(leading):
-                            yield _Segment(
-                                _box.get_row(widths, "mid", edge=show_edge),
-                                border_style,
-                            )
+                        yield _Segment(
+                            _box.get_row(widths, "mid", edge=show_edge) * leading,
+                            border_style,
+                        )
                     else:
                         yield _Segment(
                             _box.get_row(widths, "row", edge=show_edge), border_style
@@ -781,10 +857,24 @@ if __name__ == "__main__":  # pragma: no cover
     table.add_column("Title", style="magenta")
     table.add_column("Box Office", justify="right", style="green")
 
-    table.add_row("Dec 20, 2019", "Star Wars: The Rise of Skywalker", "$952,110,690")
+    table.add_row(
+        "Dec 20, 2019",
+        "Star Wars: The Rise of Skywalker",
+        "$952,110,690",
+    )
     table.add_row("May 25, 2018", "Solo: A Star Wars Story", "$393,151,347")
-    table.add_row("Dec 15, 2017", "Star Wars Ep. V111: The Last Jedi", "$1,332,539,889")
-    table.add_row("Dec 16, 2016", "Rogue One: A Star Wars Story", "$1,332,439,889")
+    table.add_row(
+        "Dec 15, 2017",
+        "Star Wars Ep. V111: The Last Jedi",
+        "$1,332,539,889",
+        style="on black",
+        end_section=True,
+    )
+    table.add_row(
+        "Dec 16, 2016",
+        "Rogue One: A Star Wars Story",
+        "$1,332,439,889",
+    )
 
     def header(text: str) -> None:
         console.print()
@@ -798,7 +888,7 @@ if __name__ == "__main__":  # pragma: no cover
 
     table.expand = True
     header("expand=True")
-    console.print(table, justify="center")
+    console.print(table)
 
     table.width = 50
     header("width=50")
